/***************************************************************************
                          jeuseqbio.h  -  description
                             -------------------
    begin                : Tue Dec 12 2000
    copyright            : (C) 2000 by Olivier Langella
    email                : Olivier.Langella@pge.cnrs-gif.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// bibliothque d'objets permettant de manipuler des
// donnees sur des squences d'ADN ou d'AA
// ecrit par Olivier Langella le 6/7/98

#ifndef JEUSEQBIO_H
#define JEUSEQBIO_H

#include "sequenceadn.h"

// jeu de Squences Biologiques
class JeuSeqBio {
public :
//	
	virtual ~JeuSeqBio();
	JeuSeqBio(istream &);//fichier au format  dterminer
	JeuSeqBio(unsigned int nseq=0);
	JeuSeqBio(const JeuSeqBio&); //constructeur de copie

	virtual SequenceBio* newSequenceBio();
	virtual SequenceBio* newSequenceBio(const SequenceBio & seq);

	void ifichier(istream &, int);//fichier au format N int
	//1= FastA
	//2= EMBL
	//3= Mega
	//4= GcgMsf
	//5= Genbank
	//0 = RAW (format brut)

	void oFormat(ostream &, int) const;//sortie au format N int

	// surcharges
	JeuSeqBio operator+ (const JeuSeqBio &);
	JeuSeqBio& operator+= (const JeuSeqBio &);
	JeuSeqBio operator+ (const SequenceBio &);
	JeuSeqBio& operator<<(istream& ientree);
	const JeuSeqBio& operator= (const JeuSeqBio &);
	virtual SequenceBio* operator[] (int i) {return(_vectseq[i]);};

	void ajouter(const SequenceBio &);

	int size() const { return (_vectseq.size());};
	int getnbseq() const { return (_vectseq.size());};
	const char * gettitrestr(int i) const { return (_vectseq[i]->_titre.c_str());};
	const char * getseqstr(int i) const { if ((i < 0)||(i >= getnbseq())) return(0); return (_vectseq[i]->c_str());};
	int gettailleseq(int i) const { if ((i < 0)||(i >= getnbseq())) return(0); return (_vectseq[i]->size());};

	void setseqstr(int i, string seq) { if ((i < 0)||(i >= getnbseq())) return; _vectseq[i]->assign(seq);};
	void settitrestr(int i, string titre) { if ((i < 0)||(i >= getnbseq())) return; _vectseq[i]->_titre = titre;};

	void elimine(unsigned int i) {
		if (i >= _vectseq.size()) return;
		delete _vectseq[i];
		//if (i == _vectseq.size()-1) _vectseq.pop_back();
		//else
		_vectseq.erase(_vectseq.begin() + i);
	};
	
	void reset();
	void SwapSeq(unsigned int a, unsigned int b);

	struct Anomalie{
		// 1-> erreur de lecture
		// 2-> format de fichier non reconnu
		// 3-> ce n'est pas le bon format
		// 10 -> pb de sortie, les squences doivent avoir la mme longueur
		// 11 -> pb de sortie, ce format n'est pas disponible
		// 12 -> il n'y a aucune squence stocke
		// 13 -> pb de sortie, format non spcifi
		// 14 -> Opration sur une squence qui n'existe pas
		int le_pb;
		Anomalie (int i=0):le_pb(i){};
	};

	static int _oformat;
 	static bool iVerifFormat(istream &);//fichier a verifier

	void fnettoie(const string &);
	void fGCGchecksum () const;

protected :
	void iFastA(istream &);//fichier au format N int
	void iEMBL(istream &);//fichier au format N int
	void iMega(istream &);//fichier au format N int
	void iRaw(istream &);//fichier au format N int
	void iGCG(istream &);//fichier au format N int
	void iGenbank(istream &);//fichier au format N int
	void oFastA(ostream &) const;
	void oEMBL(ostream &) const;
	void oMega(ostream &) const;
	void oGcgMsf(ostream &) const;


	string _titre;
	unsigned long _checktotal;
	vector<SequenceBio*> _vectseq;
};

class JeuSeqADN:public JeuSeqBio {
public :
//	
	~JeuSeqADN(){};
	JeuSeqADN();
	JeuSeqADN(const JeuSeqBio &);
	JeuSeqADN(const JeuSeqADN &);
	JeuSeqADN(int);
	
	//SequenceBio& operator[] (int i) {return(*_vectseq[i]);};
	SequenceADN* operator[] (int i) {return((SequenceADN*)_vectseq[i]);};

	SequenceADN* newSequenceBio();
};

//istream& operator>>(istream& ientree,JeuSeqBio& lesseq);

ostream&  operator<<(ostream&,const JeuSeqBio&);

#endif
